@@ -1,9 +1,7 @@
-import PySimpleGUI as sg
-from abstract_utilities.class_utils import get_fun,get_all_functions_for_instance,get_all_params
+from .SimpleGuiFunctionsManager import SimpleGuiFunctionsManager,SimpleGuiFunctionsManagerSingleton,sg,ensure_nested_list,make_component,get_gui_fun
 from abstract_utilities.path_utils import get_current_path
-from abstract_utilities.list_utils import ensure_nested_list,make_list_add,find_original_case
-from abstract_utilities import get_closest_match_from_list,eatAll
-from abstract_utilities import HistoryManager
+from abstract_utilities.list_utils import make_list_add
+from abstract_utilities import eatAll
 """
 These functions are designed to simplify and streamline the process of creating and managing PySimpleGUI windows and their layouts. The utility functions allow for more concise code when setting up GUIs.
 1. **ensure_nested_list(obj)**
@@ -71,115 +69,8 @@ These functions are designed to simplify and streamline the process of creating
 
 These functions are designed to simplify and streamline the process of creating and managing PySimpleGUI windows and their layouts. The utility functions allow for more concise code when setting up GUIs.
 """
-class WindowGlobalBridge:
-    """
-    A class to manage the global variables shared between different scripts.
 
-    Attributes:
-        global_vars (dict): A dictionary to store global variables for each script.
-
-    Methods:
-        __init__(self):
-            Initializes the WindowGlobalBridge with an empty dictionary for global_vars.
-
-        retrieve_global_variables(self, script_name, global_variables, tag_script_name=False):
-            Stores the global variables of a script in the global_vars dictionary.
-
-        return_global_variables(self, script_name=None):
-            Returns the global variables of a script.
-
-        change_globals(self, variable, value, script_name=None):
-            Modifies a global variable value for a specified script.
-
-        search_globals_values(self, value, script_name=None):
-            Searches for a specific value in the global variables of a script.
-
-        return_global_value(self, variable, script_name=None):
-            Returns the value of a specific global variable in a script.
-    """
-    def __init__(self):
-        """
-        Initializes the WindowGlobalBridge with an empty dictionary for global_vars.
-        """
-        self.global_vars = {}
-    def create_script_name(self,script_name:str='default_script_name'):
-        if script_name in self.global_vars:
-            script_name = script_name+'_0'
-        while script_name in self.global_vars:
-            script_number = int(script_name.split('_')[-1])
-            scrript_name = script_name[:-len(str(script_number))]+str(script_number+1)
-        return script_name
-    def retrieve_global_variables(self, script_name:str, global_variables:dict, tag_script_name:bool=False):
-        """
-        Stores the global variables of a script in the global_vars dictionary.
-
-        Args:
-            script_name (str): The name of the script.
-            global_variables (dict): The global variables to store for the script.
-            tag_script_name (bool, optional): If True, the script_name will be stored in the global_variables dictionary.
-                                              Defaults to False.
-        """
-        self.global_vars[script_name] = global_variables
-        if tag_script_name:
-            self.global_vars[script_name]["script_name"] = script_name
-
-    def return_global_variables(self, script_name=None):
-        """
-        Returns the global variables of a script.
-
-        Args:
-            script_name (str, optional): The name of the script. If None, all global variables will be returned.
-
-        Returns:
-            dict: The global variables of the script. If no global variables are found, it returns an empty dictionary.
-        """
-        if script_name is not None:
-            return self.global_vars.get(script_name, {})
-        else:
-            return self.global_vars
-
-    def change_globals(self, variable:str, value:any, script_name:str=None):
-        """
-        Modifies a global variable value for a specified script.
-
-        Args:
-            variable (str): The name of the global variable to modify.
-            value (any): The new value to assign to the global variable.
-            script_name (str, optional): The name of the script. If None, the global variable in the base context will be modified.
-        """
-        if script_name is not None:
-            self.global_vars[script_name][variable] = value
-            return value
-    def search_globals_values(self, value:any, script_name:str=None):
-        """
-        Searches for a specific value in the global variables of a script.
-
-        Args:
-            value (any): The value to search for in the global variables.
-            script_name (str, optional): The name of the script. If None, the search will be performed in the base context.
-
-        Returns:
-            str or False: The name of the first global variable containing the given value, or False if not found.
-        """
-        if script_name is not None:
-            for each in self.global_vars[script_name].keys():
-                if self.global_vars[script_name][each] == value:
-                    return each
-        return False
-
-    def return_global_value(self, variable:str, script_name:str=None):
-        """
-        Returns the value of a specific global variable in a script.
-
-        Args:
-            variable (str): The name of the global variable to retrieve.
-            script_name (str, optional): The name of the script. If None, the global variable in the base context will be retrieved.
-
-        Returns:
-            any: The value of the specified global variable.
-        """
-        if script_name is not None and variable in self.global_vars[script_name]:
-            return self.global_vars[script_name][variable]
+#nested list management
 def create_row(*args):
     """
     Create a row layout containing the provided arguments.
@@ -191,6 +82,7 @@ def create_row(*args):
         list: A row layout containing the provided elements.
     """
     return [arg for arg in args]
+
 def create_column(*args):
     """
     Create a column layout containing the provided arguments.
@@ -223,6 +115,7 @@ def concatenate_rows(*args):
     for arg in args:
         result += arg
     return result
+
 def concatenate_layouts(*args):
     """
     Concatenate multiple layouts into a single layout.
@@ -233,7 +126,8 @@ def concatenate_layouts(*args):
     Returns:
         list: A layout containing concatenated elements from input layouts.
     """
-    return concatenate_lists(args)
+    return concatenate_rows(args)
+
 def create_row_of_buttons(*args):
     """
     Create a row layout containing buttons generated from the provided arguments.
@@ -300,6 +194,7 @@ def get_buttons(*args):
     else:
         raise ValueError("Unsupported argument type: {}".format(arg_type))
 
+#window management
 def if_not_window_make_window(window):
     """
     Checks if the provided object is a window and creates a new window if it isn't.
@@ -316,6 +211,25 @@ def if_not_window_make_window(window):
                 window["layout"]=ensure_nested_list(window["layout"])
         window=get_window(args=window)
     return window
+      
+def create_window_manager(script_name='default_script_name',global_var=globals()):
+    """
+    Initializes a window manager for a given script.
+    
+    Args:
+        script_name (str, optional): The name of the script.
+        global_var (dict, optional): The global variables associated with the script.
+        
+    Returns:
+        tuple: A tuple containing the WindowManager, bridge, and script name.
+    """
+    bridge = WindowGlobalBridge()
+    script_name = bridge.create_script_name(script_name)
+    global_var[script_name] = script_name
+    js_bridge = bridge.retrieve_global_variables(script_name, global_var)
+    return WindowManager(script_name, bridge),bridge,script_name
+
+#event management
 def while_quick(window,return_events:(list or str)=[],exit_events:(list or str)=[sg.WIN_CLOSED],event_return=False):
     """
     Reads events from the given window and handles them based on the provided conditions.
@@ -344,7 +258,44 @@ def while_quick(window,return_events:(list or str)=[],exit_events:(list or str)=
     if event_return == True:
         return event
     return values  
-      
+
+def Choose_RPC_Parameters_GUI(RPC_list:list=None) -> dict or None:
+    """
+    Creates and launches the GUI window for selecting RPC parameters.
+    
+    Parameters:
+    - RPC_list (list, optional): The list of RPC parameters. If not provided, it will fetch the default list.
+    
+    Returns:
+    - dict: A dictionary containing the selected RPC parameters.
+    """
+    if RPC_list == None:
+        RPC_list = get_rpc_list()
+    elif os.path.isfile(RPC_list):
+        RPC_list = get_rpc_list(file_path)
+    rpc_add_global_bridge["get_rpc_list"]=[]
+    for each in RPC_list:
+        rpc_add_global_bridge["get_rpc_list"].append(RPCData(each).return_rpc_js())
+    save_rpc_list(json_data = rpc_add_global_bridge["get_rpc_list"])
+    rpc_add_global_bridge["total_bool_list"] = []
+    rpc_add_global_bridge["recursed_rpc_js_list"]=rpc_add_global_bridge["get_rpc_list"] 
+    rpc_add_global_bridge["check_list"]={}
+    for each in list(get_js().values()):
+        rpc_add_global_bridge["check_list"][each] = ''
+    keyed_rpc_lists()
+    rpc_add_global_bridge["Network_Names"]=rpc_add_global_bridge["keyed_lists"]["Network_Name"]
+    layout = []
+    for key,value in get_js().items():
+        layout.append([
+            get_gui_fun("Text",args={"text":key.replace('_',' ')}),
+            get_gui_fun("Combo",args={"values":rpc_add_global_bridge["keyed_lists"][key],"default_text":rpc_add_global_bridge["keyed_lists"][key][0],"key":f"{value}","enable_events":True}),
+            get_gui_fun("Checkbox",args={"text":"","default":(key=="Network_Name"),"key":f"{value[:-1]}_CHECK-","enable_events":True}),
+            get_push()])
+    layout = [[get_menu()],layout,[create_row_of_buttons("OK","Show","reset","Exit"),]]
+    window = window_mgr.get_new_window(args={"title":'ADD RPC',"layout":layout,"exit_events":["OK","Exit"],"event_function":"win_while","suppress_raise_key_errors":False, "suppress_error_popups":False, "suppress_key_guessing":False,"finalize":True})
+    values = window_mgr.while_basic(window=window)
+
+#premade components
 def verify_args(args:dict=None, layout:list=None, title:str=None, event_function:str=None,exit_events:(list or str)=None):
     """
     Verifies and/or sets default values for window arguments.
@@ -509,80 +460,60 @@ def get_browser(title:str=None,type:str='Folder',args:dict={},initial_folder:str
     if results == None:
         results={'output':initial_folder}
     return results['output']
-def get_gui_fun(function_name: str = '', args: dict = {}):
-    """
-    Returns a callable object for a specific PySimpleGUI function with the provided arguments.
-
-    Args:
-        function_name (str): The name of the PySimpleGUI function.
-        *args_2: Variable-length positional arguments.
-        args (dict): The arguments to pass to the PySimpleGUI function.
-        **kwargs: Variable-length keyword arguments.
-
-    Returns:
-        callable: A callable object that invokes the PySimpleGUI function with the specified arguments when called.
-    """
-    return get_fun({"instance": sg, "name": function_name, "args": args})
-
-def agf(function_name: str = '', *args_2, args: dict = {}, **kwargs):
-    """
-    Returns a callable object for a specific PySimpleGUI function with the provided arguments.
-
-    Args:
-        function_name (str): The name of the PySimpleGUI function.
-        *args_2: Variable-length positional arguments.
-        args (dict): The arguments to pass to the PySimpleGUI function.
-        **kwargs: Variable-length keyword arguments.
-
-    Returns:
-        callable: A callable object that invokes the PySimpleGUI function with the specified arguments when called.
-    """
-    instance = sg
-    functions = get_all_functions_for_instance(instance=instance)
-    if function_name not in functions:
-        name = find_original_case(functions, function_name)
-        if name is None:
-            name = get_closest_match_from_list(comp_str=function_name, total_list=functions)
-        print(f"{function_name} not found, changing to closest match: {name}")
-        function_name = name
-    
-    all_params = get_all_params(instance=instance, function_name=function_name)
-    required_params = all_params["names"]
-    # Map positional arguments to their respective required_params
-    for i, param in enumerate(required_params):
-        if i < len(args_2) and param not in args:
-            args[param] = args_2[i]
-    for param in required_params:
-        if param not in args:
-            change = None
-            if param == "title":
-                change = ""
-            if param == "layout":
-                change = [[]]
-            if param == "values":
-                change = []
-            if param == "text":
-                change = ""
-            if change is not None:
-                if function_name != 'Window':
-                    print(f"{param} for {function_name} not in arguments, inputting {param} = {change}")
-                args[param] = change
-    if "layout" in all_params["names"]:
-        args["layout"] = ensure_nested_list(args.get("layout", []))
-    
-    return get_fun({"instance": sg, "name": function_name, "args": args})
 
 def get_menu(menu_definition:list=[['File',  'Save', 'Exit',],['Edit', ['Paste', ['Special', 'Normal',], 'Undo'],],['Help', 'About...'],],args:dict={}):
     args["menu_definition"]=ensure_nested_list(menu_definition)
     return get_gui_fun("Menu",args=args)
-def get_push()-> get_gui_fun("Push"):
+
+def get_push()-> make_component("Push"):
     """
     Fetches the "Push" function from the GUI module.
     
     Returns:
     - function: The "Push" GUI function.
     """
-    return get_gui_fun("Push")
+    return make_component("Push")
+
+#key management
+def text_to_key(text,section=None):
+    text=eatAll(text,['-'])
+    if section != None:
+        text=f"{text} {section}"
+    return f"-{text.upper().replace(' ','_')}-"
+
+def get_event_key_js(event,key_list=None):
+    script_event_js = {"event":event,'found':event,'section':''}
+    if event:
+        section,found=None,None
+        if event in key_list:
+            found=event
+        else:
+            section = eatAll(event,['-']).split('_')[-1]
+        script_event_js = {"event":event,'found':found,'section':section}
+        for script_event in key_list:
+            conversion = text_to_key(script_event,section=section)
+            script_event_js[script_event]=conversion
+            if event == conversion:
+                script_event_js['found']=script_event
+    return script_event_js
+
+#size parameters
+def get_screen_size():
+    """
+    This function gets the screen size. No input arguments. Returns the screen size gotten from the sg.Window class.
+    """
+    return make_component("Window",title='screen_size',layout=[[]]).get_screen_size()
+
+def get_screen_dimensions(height:int,width:int)->(int,int):
+    screen_width, screen_height = get_screen_size()
+    # Ensure we have valid screen dimensions
+    if screen_width:
+        screen_width=int(screen_width * width)
+    if screen_height:
+        screen_height=int(screen_height * height)
+    if screen_width is None or screen_height is None:
+        raise ValueError("Could not determine screen dimensions.")
+    return screen_width, screen_height
 def expandable(size: tuple = (None, None),
                resizable: bool = True,
                scroll_vertical: bool = False,
@@ -602,98 +533,3 @@ def expandable(size: tuple = (None, None),
         "expand_x": expand_x,
         "expand_y": expand_y
     }
-def create_window_manager(script_name='default_script_name',global_var=globals()):
-    """
-    Initializes a window manager for a given script.
-    
-    Args:
-        script_name (str, optional): The name of the script.
-        global_var (dict, optional): The global variables associated with the script.
-        
-    Returns:
-        tuple: A tuple containing the WindowManager, bridge, and script name.
-    """
-    bridge = WindowGlobalBridge()
-    script_name = bridge.create_script_name(script_name)
-    global_var[script_name] = script_name
-    js_bridge = bridge.retrieve_global_variables(script_name, global_var)
-    return WindowManager(script_name, bridge),bridge,script_name
-def Choose_RPC_Parameters_GUI(RPC_list:list=None) -> dict or None:
-    """
-    Creates and launches the GUI window for selecting RPC parameters.
-    
-    Parameters:
-    - RPC_list (list, optional): The list of RPC parameters. If not provided, it will fetch the default list.
-    
-    Returns:
-    - dict: A dictionary containing the selected RPC parameters.
-    """
-    if RPC_list == None:
-        RPC_list = get_rpc_list()
-    elif os.path.isfile(RPC_list):
-        RPC_list = get_rpc_list(file_path)
-    rpc_add_global_bridge["get_rpc_list"]=[]
-    for each in RPC_list:
-        rpc_add_global_bridge["get_rpc_list"].append(RPCData(each).return_rpc_js())
-    save_rpc_list(json_data = rpc_add_global_bridge["get_rpc_list"])
-    rpc_add_global_bridge["total_bool_list"] = []
-    rpc_add_global_bridge["recursed_rpc_js_list"]=rpc_add_global_bridge["get_rpc_list"] 
-    rpc_add_global_bridge["check_list"]={}
-    for each in list(get_js().values()):
-        rpc_add_global_bridge["check_list"][each] = ''
-    keyed_rpc_lists()
-    rpc_add_global_bridge["Network_Names"]=rpc_add_global_bridge["keyed_lists"]["Network_Name"]
-    layout = []
-    for key,value in get_js().items():
-        layout.append([
-            get_gui_fun("Text",args={"text":key.replace('_',' ')}),
-            get_gui_fun("Combo",args={"values":rpc_add_global_bridge["keyed_lists"][key],"default_text":rpc_add_global_bridge["keyed_lists"][key][0],"key":f"{value}","enable_events":True}),
-            get_gui_fun("Checkbox",args={"text":"","default":(key=="Network_Name"),"key":f"{value[:-1]}_CHECK-","enable_events":True}),
-            get_push()])
-    layout = [[get_menu()],layout,[create_row_of_buttons("OK","Show","reset","Exit"),]]
-    window = window_mgr.get_new_window(args={"title":'ADD RPC',"layout":layout,"exit_events":["OK","Exit"],"event_function":"win_while","suppress_raise_key_errors":False, "suppress_error_popups":False, "suppress_key_guessing":False,"finalize":True})
-    values = window_mgr.while_basic(window=window)
-def make_component(function_name,*arges,**kwargs):
-    functions = get_all_functions_for_instance(instance=sg)
-    if function_name not in functions:
-        if function_name !=  None:
-            function_name = find_original_case(functions, function_name)
-    if function_name == None or function_name not in functions:
-        name=None
-        for each in arges:
-            print(each)
-            if function_name not in functions and function_name != None:
-                name = find_original_case(functions, function_name)
-        if name is None:
-            for fun_name in arges:
-                name = get_closest_match_from_list(comp_str= arges, total_list=functions)
-                print(f"{function_name} not found, changing to closest match: {name}")
-                function_name = name
-        else:
-            function_name=name
-    if 'args' in kwargs:
-        args=kwargs['args']
-        del kwargs['args']
-        
-        kwargs.update(args)
-    return agf(function_name,*arges,args=kwargs)
-def text_to_key(text,section=None):
-    text=eatAll(text,['-'])
-    if section != None:
-        text=f"{text} {section}"
-    return f"-{text.upper().replace(' ','_')}-"
-def get_event_key_js(event,key_list=None):
-    script_event_js = {"event":event,'found':event,'section':''}
-    if event:
-        section,found=None,None
-        if event in key_list:
-            found=event
-        else:
-            section = eatAll(event,['-']).split('_')[-1]
-        script_event_js = {"event":event,'found':found,'section':section}
-        for script_event in key_list:
-            conversion = text_to_key(script_event,section=section)
-            script_event_js[script_event]=conversion
-            if event == conversion:
-                script_event_js['found']=script_event
-    return script_event_js
